----------------------------------------------------------------------------------
-- Company: 
-- Engineer: 
-- 
-- Create Date: 09/02/2015 05:04:45 PM
-- Design Name: 
-- Module Name: my_counter - Behavioral
-- Project Name: 
-- Target Devices: 
-- Tool Versions: 
-- Description: 
-- 
-- Dependencies: 
-- 
-- Revision:
-- Revision 0.01 - File Created
-- Additional Comments:
-- 
----------------------------------------------------------------------------------


library IEEE;
use IEEE.STD_LOGIC_1164.ALL;

-- Uncomment the following library declaration if using
-- arithmetic functions with Signed or Unsigned values
use IEEE.NUMERIC_STD.ALL;

-- Uncomment the following library declaration if instantiating
-- any Xilinx leaf cells in this code.
--library UNISIM;
--use UNISIM.VComponents.all;

entity my_counter is
    Port ( start_writes : in STD_LOGIC;
           start_reads  : in STD_LOGIC;
           rst : in STD_LOGIC;
           clk_in1 : in STD_LOGIC;
           read_port_val : out unsigned (7 downto 0));
end my_counter;

architecture Behavioral of my_counter is

component clk_wiz_0
port
 (-- Clock in ports
  clk_in1           : in     std_logic;
  -- Clock out ports
  clk_out1          : out    std_logic;
  -- Status and control signals
  reset             : in     std_logic;
  locked            : out    std_logic
 );
end component;

component ila_0 
PORT (
clk : IN STD_LOGIC;
probe0 : IN STD_LOGIC_VECTOR(37 DOWNTO 0)
);
end component;

component blk_mem_gen_0 IS
  PORT (
    clka : IN STD_LOGIC;
    ena : IN STD_LOGIC;
    wea : IN STD_LOGIC_VECTOR(0 DOWNTO 0);
    addra : IN STD_LOGIC_VECTOR(7 DOWNTO 0);
    dina : IN STD_LOGIC_VECTOR(7 DOWNTO 0);
    clkb : IN STD_LOGIC;
    enb : IN STD_LOGIC;
    addrb : IN STD_LOGIC_VECTOR(7 DOWNTO 0);
    doutb : OUT STD_LOGIC_VECTOR(7 DOWNTO 0)
  );
END component;


signal  clk_out1    : std_logic;
signal  locked      : std_logic;
signal  local_count : unsigned (19 downto 0);
signal  local_probe : std_logic_vector(37 downto 0);
signal  local_count_val : std_logic_vector(3 downto 0);

signal write_address_port_a: unsigned(7 downto 0);
signal read_address_port_b: unsigned(7 downto 0);

signal write_data_port_a: std_logic_vector(7 downto 0);
signal read_data_port_a : std_logic_vector(7 downto 0);

begin


  my_clock_gen: clk_wiz_0
   port map ( 
  -- Clock in ports
  clk_in1 => clk_in1,
 -- Clock out ports  
  clk_out1 => clk_out1,
 -- Status and control signals                
  reset => '0',
  locked => locked            
);

  my_ila : ila_0
  port  map(
    clk => clk_out1,
    probe0 => local_probe);

local_probe <= std_logic_vector(write_address_port_a) & "00" & std_logic_vector(read_address_port_b) & "00" & write_data_port_a & "00" & read_data_port_a;

process(clk_out1, rst)
begin
  if(rst = '0') then
    write_address_port_a    <= (others => '0');
    read_address_port_b     <= (others => '0');
    write_data_port_a       <= (others => '0');
  elsif(rising_edge(clk_out1)) then
    if(start_writes = '1') then 
        write_data_port_a    <= std_logic_vector(unsigned(write_address_port_a) + 10);
        write_address_port_a <= ((write_address_port_a) + 1);
    else
        write_data_port_a    <= write_data_port_a;
        write_address_port_a <= write_address_port_a;
    end if;      
    if(start_reads = '1') then 
        read_address_port_b <= ((read_address_port_b) + 1);
    else
        read_address_port_b <= read_address_port_b;
    end if;        
  end if;
 end process;
 
 
 
 -- instantiate block mem
 my_mem_inst :blk_mem_gen_0 
   PORT MAP(
     clka   => clk_out1,
     ena    => '1',
     wea    => "1",
     addra  => std_logic_vector(write_address_port_a),
     dina   => write_data_port_a,
     clkb   => clk_out1,
     enb    => '1',
     addrb  => std_logic_vector(read_address_port_b),
     doutb  => read_data_port_a
   );


--assign output port
read_port_val <= unsigned(read_data_port_a);


    
end Behavioral;
